C=============================================================================C
C International Comprehensive Ocean-Atmosphere Data Set (ICOADS)   6 Jun 2005 C
C Filename:level: trimqc2.f:01C                            Fortran 77 library C
C Purpose: Use QC flags from IMMA                    Author: S. Worley et al. C
C=============================================================================C
C Software Revision Information (previous version: 19 May 2005, level 01B):
C Minor corrections and clarifications of comments (no code changes).
C-----------------------------------------------------------------------3456789
C Software documentation for the (invariant) user-interface routine {trimqc2}:
C
C Functionality:  This routine is called from within {rdimma0} after the call
C to {getrpt} (see additional comments below).  It uses the trimming and other
C QC flags to reject entire reports, or to reject individual data elements
C within the reports.  Six of the input arguments provide extensive options
C for using, or ignoring, the QC flags.  Subroutine {trimqc0.f} is required
C and also contains documentation for the option settings.  Note: longitudes
C must conform to the ICOADS convention, or a STOP indicating:
C      TRIMQC0 ERROR. TRIMMING FLAGS MISSING
C will occur in the event of any negative longitudes (NCDC convention).
C
C External library required: {trimqc0.f}. 
C Machine dependencies:  None known.
C For more information: See <imma> and <stat_trim> (electronic documents).
C=============================================================================C
C WARNING:  Code beyond this point should not require any modification.       C
C=============================================================================C
C-----------------------------------------------------------------------3456789
      SUBROUTINE GETTRF(ITRUE,QCFLG,MISS,ERR
     +,LAT,LON,D,W,B10,ND,SF,AF,UF,VF,PF,RF,QCE,LZ,QCZ,TRFLG)
C GET TRIMMING FLAGS
      IMPLICIT INTEGER(A-E,G-Z)
      PARAMETER(ZNC=1,WNC=2,BNC=3,XNC=4,YNC=5,PNC=6,ANC=7,GNC=8,DNC=9
     +,SNC=10,CNC=11,ENC=12,FNC=13,TNC=14)
      DIMENSION ITRUE(*),QCFLG(14),TRFLG(23)
      LOGICAL B2QXY
C
      DO 190 I=1,23
        TRFLG(I)=0
  190 CONTINUE
      IF (ITRUE(LAT).NE.MISS .AND. ITRUE(LAT).NE.ERR .AND.
     +    ITRUE(LON).NE.MISS .AND. ITRUE(LON).NE.ERR .AND.
     +    ITRUE(B10).NE.MISS .AND. ITRUE(B10).NE.ERR) THEN
        IF (B2QXY(QB10(ITRUE(B10)),ITRUE(LON),ITRUE(LAT),B2))
     +  TRFLG( 1)=B2
      ENDIF
      IF (ITRUE(ND).NE.MISS) THEN
        TRFLG( 2)=ITRUE(ND)
      ENDIF
      IF (ITRUE(SF).NE.MISS) THEN
        TRFLG( 3)=ITRUE(SF)
        TRFLG( 4)=ITRUE(AF)
        TRFLG( 5)=ITRUE(UF)
        TRFLG( 6)=ITRUE(VF)
        TRFLG( 7)=ITRUE(PF)
        TRFLG( 8)=ITRUE(RF)
      ENDIF
      IF (QCFLG(ZNC).NE.MISS) THEN
        IF (QCFLG(ZNC).GE.7 .AND. QCFLG(ZNC).NE.10) TRFLG( 9)=1
        IF (QCFLG(SNC).GE.8 .AND. QCFLG(SNC).NE.10) TRFLG(10)=1
        IF (QCFLG(ANC).GE.8 .AND. QCFLG(ANC).NE.10) TRFLG(11)=1
        IF (ITRUE(D).GE.1 .AND. ITRUE(D).LE.360 .AND. ITRUE(W).EQ.0
     +                      .AND. QCFLG(WNC).EQ. 7) TRFLG(12)=1
        IF (QCFLG(PNC).GE.8 .AND. QCFLG(PNC).NE.10) TRFLG(13)=1
        IF (QCFLG(GNC).GE.8 .AND. QCFLG(GNC).NE.10) TRFLG(14)=1
        IF (QCFLG(DNC).GE.8 .AND. QCFLG(DNC).NE.10) TRFLG(14)=1
        IF (QCFLG(XNC).GE.2 .AND. QCFLG(XNC).NE.10) TRFLG(15)=1
        IF (QCFLG(XNC).GE.4 .AND. QCFLG(XNC).NE.10) TRFLG(15)=2
        IF (QCFLG(XNC).GE.7 .AND. QCFLG(XNC).NE.10) TRFLG(15)=3
        IF (QCFLG(CNC).GE.2 .AND. QCFLG(CNC).NE.10) TRFLG(16)=1
        IF (QCFLG(CNC).GE.4 .AND. QCFLG(CNC).NE.10) TRFLG(16)=2
        IF (QCFLG(CNC).GE.7 .AND. QCFLG(CNC).NE.10) TRFLG(16)=3
        IF (QCFLG(ENC).GE.2 .AND. QCFLG(ENC).NE.10) TRFLG(17)=1
        IF (QCFLG(ENC).GE.4 .AND. QCFLG(ENC).NE.10) TRFLG(17)=2
        IF (QCFLG(ENC).GE.7 .AND. QCFLG(ENC).NE.10) TRFLG(17)=3
      ENDIF
      IF (ITRUE(QCE).NE.MISS) THEN
        TRFLG( 9)=TRFLG( 9)+2*MOD(ITRUE(QCE)/2/2/2/2/2,2)
        TRFLG(10)=TRFLG(10)+2*MOD(ITRUE(QCE)/2/2/2/2,2)
        TRFLG(11)=TRFLG(11)+2*MOD(ITRUE(QCE)/2/2/2,2)
        TRFLG(12)=TRFLG(12)+2*MOD(ITRUE(QCE)/2/2,2)
        TRFLG(13)=TRFLG(13)+2*MOD(ITRUE(QCE)/2,2)
        TRFLG(14)=TRFLG(14)+2*MOD(ITRUE(QCE),2)
      ENDIF
      IF (ITRUE(LZ).NE.MISS) THEN
        TRFLG(18)=ITRUE(LZ)
      ENDIF
      IF (ITRUE(QCZ).NE.MISS) THEN
        TRFLG(19)=MOD(ITRUE(QCZ)/2/2/2/2,2)
        TRFLG(20)=MOD(ITRUE(QCZ)/2/2/2,2)
        TRFLG(21)=MOD(ITRUE(QCZ)/2/2,2)
        TRFLG(22)=MOD(ITRUE(QCZ)/2,2)
        TRFLG(23)=MOD(ITRUE(QCZ),2)
      ENDIF
      END
C$QB10------------------------------------------------------------------3456789
      FUNCTION QB10(B10)
      IMPLICIT INTEGER(A-E,G-Z)
      QB10=-1
      IF(B10.LT.1.OR.B10.GT.648)RETURN
      QB10=2+(B10-1)/324*2-MOD(B10-1+3,36)/18
      END
C$B2QXY-----------------------------------------------------------------3456789
      LOGICAL FUNCTION B2QXY(Q,X,Y,B2)
      IMPLICIT INTEGER(A-E,G-Z)
C
      B2QXY=.FALSE.
      IF(Q.LT.1.OR.Q.GT.4.OR.X.LT.0.OR.X.GT.35999.OR.ABS(Y).GT.9000)
     +RETURN
C
      IF(ABS(Y).LT.9000)THEN
        IF(MOD(Q,2).EQ.0)THEN
          C=MIN(X/200,89)
        ELSE
          C=179-MIN(MOD(36000-X,36000)/200,89)
        ENDIF
        IF(Q/3.EQ.0)THEN
          R=89-(9000+Y)/200
        ELSE
          R=(9000-Y)/200
        ENDIF
        B2=2+R*180+C
C
      ELSE IF(Y.EQ.9000)THEN
        B2=1
C
      ELSE
        B2=16202
      ENDIF
C
      B2QXY=.TRUE.
      END
